@@ -1,17 +1,43 @@
-use crate::Indices;
+use crate::{Indices};
 use crate::here;
 
-/// New reversed generic vector
+/// Reverse a generic slice by reverse iteration.
+/// Creates a new Vec (is immutable).
+/// Its naive use for descending sort etc. is to be avoided for
+/// efficiency reasons. Included here just for convenience.
 pub fn revs<T>(s: &[T]) -> Vec<T> where T: Copy, 
-   { s.iter().rev().map(|&x| x).collect::<Vec<T>>() }
+    { s.iter().rev().map(|&x| x).collect::<Vec<T>>() }
+
+/// Binary search of a sorted list (in ascending order).
+/// Returns the index of the first item that is greater than val. 
+/// When none are greater, returns s.len() (invalid index but logical).
+/// Example use: looking up cummulative probability density functions. 
+pub fn binsearch<T>(s:&[T], val: T)  -> usize where T: PartialOrd, {     
+    let n = s.len();
+    if n < 2 { panic!("{} vec of data is too short!",here!()) }     
+    if s[0] > val { return 0_usize }; // the first item already exceeds val
+    let mut hi = n-1; // valid index of the last item
+    if s[hi] <= val { return n }; // no items exceed val
+    let mut lo = 0_usize; // initial index of the low limit     
+    loop {
+        let gap = hi - lo;
+        if gap <= 1 { return hi }
+        let tryi = lo+gap/2; 
+        // if tryi's value is above val, reduce the high index to it
+        if s[tryi] > val { hi = tryi; continue }            
+        // else tryi's value is not greater than val, raise the low index to it
+        // jumps also over any repeated equal values. 
+        lo = tryi
+    }  
+}
 
 /// Merges two ascending sorted generic vectors,
 /// by classical selection and copying of their head items into the result.
 /// Consider using merge_indexed instead, especially for non-primitive end types T. 
 pub fn merge<T>(v1: &[T], v2: &[T]) -> Vec<T> where T: PartialOrd+Copy, {  
-    let mut resvec:Vec<T> = Vec::new();
     let l1 = v1.len();
     let l2 = v2.len();
+    let mut resvec:Vec<T> = Vec::with_capacity(l1+l2);
     let mut i1 = 0;
     let mut i2 = 0;
     loop {
@@ -34,32 +60,33 @@ pub fn merge<T>(v1: &[T], v2: &[T]) -> Vec<T> where T: PartialOrd+Copy, {
 
 /// Merges two ascending sort indices.
 /// Data is not shuffled at all, v2 is just concatenated onto v1
-/// in one go and both remain in their original order.
+/// in one go and both remain in their original order. 
 /// Returns the concatenated vector and a new valid sort index into it.
 pub fn merge_indexed<T>(v1:&[T], idx1: &[usize], v2: &[T], idx2: &[usize]) -> ( Vec<T>,Vec<usize> ) 
-    where T: PartialOrd+Copy,    
-{     
-    let res = [v1,v2].concat(); // no individual shuffling, just bulk concatenation     
+    where T: PartialOrd+Copy, {    
+    let res = [v1,v2].concat(); // no individual shuffling, just one concatenation     
     let l = idx1.len();
     // shift up all items in idx2 by length of indx1, so that they will 
     // refer correctly to the second part of the concatenated vector
     let idx2shifted:Vec<usize> = idx2.iter().map(|x| l+x ).collect();
-    // now perform the merge of the indices only     
+    // now merge the indices      
     let residx = merge_indices(&res,idx1,&idx2shifted);   
     ( res, residx )
 }
 
-/// Merges the sort indices of two simply concatenated vectors.
+/// Merges the sort indices of two concatenated vectors.
 /// Data in s is not changed at all, only consulted for the comparisons. 
-/// Used by  `mergesort` and `merge_indexed`. 
+/// This function is used by  `mergesort` and `merge_indexed`. 
 fn merge_indices<T>(s: &[T], idx1:&[usize], idx2:&[usize]) -> Vec<usize>
     where T: PartialOrd+Copy, {
     let l1 = idx1.len();
     let l2 = idx2.len();
-    let mut residx:Vec<usize> = Vec::new(); 
-    let mut i1 = 0;  let mut i2 = 0;
-        let mut head1 = s[idx1[i1]]; let mut head2 = s[idx2[i2]];
-        loop {
+    let mut residx:Vec<usize> = Vec::with_capacity(l1+l2); 
+    let mut i1 = 0;  
+    let mut i2 = 0;
+    let mut head1 = s[idx1[i1]]; 
+    let mut head2 = s[idx2[i2]];
+    loop {
         if head1 < head2 { 
             residx.push(idx1[i1]);
             i1 += 1;  
@@ -99,12 +126,12 @@ fn merge_indices<T>(s: &[T], idx1:&[usize], idx2:&[usize]) -> Vec<usize>
     residx
 }
 
-/// Doubly recursive non-destructive merge sort. 
-/// The data is read-only, it is not moved or mutated.
+/// Doubly recursive non-destructive merge sort.  
+/// The data is not moved or mutated. 
 /// Efficiency is comparable to quicksort. 
-/// Returns a vector of indices to self from i to i+n,
-/// such that the indexed values are in sort order (sort index).  
-/// Thus only the index values are being moved. 
+/// Returns a vector of indices to s from i to i+n,
+/// such that the indexed values are in ascending sort order (a sort index).  
+/// Only the index values are being moved. 
 fn mergesort<T>(s:&[T], i:usize, n:usize) -> Vec<usize> 
     where T: PartialOrd+Copy {
     if n == 1 { let res = vec![i]; return res };  // recursion termination
@@ -115,27 +142,29 @@ fn mergesort<T>(s:&[T], i:usize, n:usize) -> Vec<usize>
     let n2 = n - n1; // the remaining second part
     let sv1 = mergesort(s, i, n1); // recursively sort the first half
     let sv2 = mergesort(s, i+n1, n2); // recursively sort the second half 
-    // Now merge the two sorted indices into one      
+    // Now merge the two sorted indices into one and return it     
     merge_indices(s,&sv1,&sv2)
 }
 
-/// A simple wrapper for mergesort, when we want just the sort index
-/// of the entire input vector.
+/// A wrapper for mergesort, to obtain the sort index
+/// of the (whole) input vector. Simpler than sortm.
 pub fn sortidx<T>(s:&[T]) -> Vec<usize> where T:PartialOrd+Copy {
     mergesort(&s,0,s.len())
 }
 
-/// Immutable sort. Returns new sorted vector (ascending or descending)
-/// Simple wrapper for mergesort. Passes the boolean flag 'ascending' onto 'unindex'.
+/// Immutable sort. Returns new sorted vector (ascending or descending). 
+/// Is a wrapper for mergesort. Passes the boolean flag 'ascending' onto 'unindex'.
+/// Mergesort by itself always produces only an ascending index.
 pub fn sortm<T>(s:&[T], ascending:bool) -> Vec<T> where T: PartialOrd+Copy {
-    mergesort(s,0,s.len()).unindex(ascending,s)
+    mergesort(s,0,s.len()).unindex(s,ascending)
 }   
 
-/// Ranking by inverting the sort index.  
-/// Sort index is in sorted order, giving indices to the original data positions.
-/// Ranking is in  original data order, giving positions in the sorted order (sort index).
-/// Thus they are in an inverse relationship, easily converted by `.invindex()`
-/// Fast ranking of many T items, with only n*(log(n)+1) complexity.
+/// Fast ranking of many T items, with only `n*(log(n)+1)` complexity. 
+/// Ranking is done by inverting the sort index.  
+/// Sort index is in sorted order, giving data positions. 
+/// Ranking is in data order, giving sorted order positions. 
+/// Thus sort index and ranks are in an inverse relationship. 
+/// They are easily converted by `.invindex()` (for: invert index).
 pub fn rank<T>(s:&[T], ascending:bool) -> Vec<usize> where T:PartialOrd+Copy {
     let n = s.len();
     let sortindex = mergesort(s,0,n);
@@ -151,27 +180,3 @@ pub fn rank<T>(s:&[T], ascending:bool) -> Vec<usize> where T:PartialOrd+Copy {
     }
     rankvec 
 }
-
-/// Returns the index of the first item that is greater than val, 
-/// using binary search of an already ascending sorted list.
-/// When all are smaller, returns self.len().
-/// When the last item is equal, returns self.len()-1.
-/// Example use: looking up cummulative probability density functions. 
-pub fn binsearch<T>(s:&[T], val: T)  -> usize where T: PartialOrd, {     
-    let n = s.len();
-    if n < 2 { panic!("{} vec of data is too short!",here!()) }     
-    if val < s[0] { return 0_usize }; // v is smaller than the first item
-    let mut hi = n-1; // valid index of the last item
-    if val > s[hi] { return n }; // indicates that v is greater than the last item
-    let mut lo = 0_usize; // initial index of the low limit     
-    loop {
-        let gap = hi - lo;
-        if gap <= 1 { return hi }
-        let tryi = lo+gap/2; 
-        // if tryi index's value is above val, reduce the high index
-        if s[tryi] > val { hi = tryi; continue }            
-        // else indexed value is not greater than v, raise the low index;
-        // jumps also repeating equal values. 
-        lo = tryi
-    }  
-} 
